@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <util/alloc.h>
+#include <util/bitarray.h>
 #include <util/exit.h>
 #include <util/gv_math.h>
 #include <util/itos.h>
@@ -33,10 +34,9 @@
 #include <util/streq.h>
 
 struct adjmatrix_t {
-  size_t nrows;
-  size_t ncols;
-  uint8_t *data;    ///< bit-packed backing memory
-  size_t allocated; ///< how many bytes have been allocated backing `data`?
+  size_t nrows;  ///< how many rows have been allocated?
+  size_t ncols;  ///< how many columns have been allocated?
+  uint8_t *data; ///< bit-packed backing memory
 };
 
 /// get the value of a matrix cell
@@ -48,14 +48,17 @@ struct adjmatrix_t {
 static bool matrix_get(adjmatrix_t *me, size_t row, size_t col) {
   assert(me != NULL);
 
-  // if this index is beyond anything set, infer it as unset
-  const size_t index = row * me->ncols + col;
-  const size_t byte_index = index / 8;
-  const size_t bit_index = index % 8;
-  if (byte_index >= me->allocated) {
+  // if this index is beyond anything allocated, infer it as unset
+  if (row >= me->nrows) {
+    return false;
+  }
+  if (col >= me->ncols) {
     return false;
   }
 
+  const size_t index = row * me->ncols + col;
+  const size_t byte_index = index / 8;
+  const size_t bit_index = index % 8;
   return (me->data[byte_index] >> bit_index) & 1;
 }
 
@@ -68,15 +71,38 @@ static void matrix_set(adjmatrix_t *me, size_t row, size_t col) {
   assert(me != NULL);
 
   // if we are updating beyond allocated space, expand the backing store
+  if (row >= me->nrows || col >= me->ncols) {
+    // allocate an enlarged space
+    const size_t nrows = zmax(me->nrows, row + 1);
+    const size_t ncols = zmax(me->ncols, col + 1);
+    const size_t bits = nrows * ncols;
+    const size_t bytes = bits / 8 + (bits % 8 == 0 ? 0 : 1);
+    uint8_t *const data = gv_alloc(bytes);
+
+    // replicate set bits
+    for (size_t r = 0; r < me->nrows; ++r) {
+      for (size_t c = 0; c < me->ncols; ++c) {
+        if (!matrix_get(me, r, c)) {
+          continue;
+        }
+        const size_t index = r * ncols + c;
+        const size_t byte_index = index / 8;
+        const size_t bit_index = index % 8;
+        data[byte_index] |= (uint8_t)(UINT8_C(1) << bit_index);
+      }
+    }
+
+    // replace old matrix with newly expanded one
+    free(me->data);
+    *me = (adjmatrix_t){.nrows = nrows, .ncols = ncols, .data = data};
+  }
+
+  assert(row < me->nrows);
+  assert(col < me->ncols);
+
   const size_t index = row * me->ncols + col;
   const size_t byte_index = index / 8;
   const size_t bit_index = index % 8;
-  if (byte_index >= me->allocated) {
-    me->data = gv_recalloc(me->data, me->allocated, byte_index + 1,
-                           sizeof(me->data[0]));
-    me->allocated = byte_index + 1;
-  }
-
   me->data[byte_index] |= (uint8_t)(UINT8_C(1) << bit_index);
 }
 
@@ -97,17 +123,27 @@ static void merge2(graph_t * g);
 static void init_mccomp(graph_t *g, size_t c);
 static void cleanup2(graph_t *g, int64_t nc);
 /// @return minimum crossings on success, negative value on failure
-static int64_t mincross_clust(graph_t *g, ints_t *scratch);
+static int64_t mincross_clust(graph_t *g);
 /// @return minimum crossings on success, negative value on failure
-static int64_t mincross(graph_t *g, int startpass, ints_t *scratch);
+static int64_t mincross(graph_t *g, int startpass);
 static void mincross_step(graph_t * g, int pass);
 static void mincross_options(graph_t * g);
 static void save_best(graph_t * g);
 static void restore_best(graph_t * g);
-static adjmatrix_t *new_matrix(size_t i, size_t j);
+
+/// create an adjacency matrix
+///
+/// These matrices dynamically expand on demand. So the initial dimensions
+/// provided can be later safely exceeded at runtime.
+///
+/// @param initial_rows How many rows to allocate now
+/// @param initial_columns How many columns to allocate now
+/// @return A constructed matrix
+static adjmatrix_t *new_matrix(size_t initial_rows, size_t initial_columns);
+
 static void free_matrix(adjmatrix_t * p);
 static int ordercmpf(const void *, const void *);
-static int64_t ncross(ints_t *scratch);
+static int64_t ncross(void);
 #ifdef DEBUG
 void check_rs(graph_t * g, int null_ok);
 void check_order(void);
@@ -147,26 +183,26 @@ static void nname(node_t *v, FILE *stream) {
 }
 static void dumpg (graph_t* g)
 {
-    int j, i, r;
-    node_t* v;
     edge_t* e;
 
     fprintf (stderr, "digraph A {\n");
-    for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
+    for (int r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 	fprintf (stderr, "  subgraph {rank=same  ");
-	for (i = 0; i < GD_rank(g)[r].n; i++) {
-	  v = GD_rank(g)[r].v[i];
-          if (i > 0)
+	const char *trailer = " }\n";
+	for (int i = 0; i < GD_rank(g)[r].n; i++) {
+	  node_t *const v = GD_rank(g)[r].v[i];
+          if (i > 0) {
  	    fputs(" -> ", stderr);
+ 	    trailer = " [style=invis]}\n";
+          }
           nname(v, stderr);
         }
-        if (i > 1) fprintf (stderr, " [style=invis]}\n");
-        else fprintf (stderr, " }\n");
+        fputs(trailer, stderr);
     }
-    for (r = GD_minrank(g); r < GD_maxrank(g); r++) {
-	for (i = 0; i < GD_rank(g)[r].n; i++) {
-	  v = GD_rank(g)[r].v[i];
-	  for (j = 0; (e = ND_out(v).list[j]); j++) {
+    for (int r = GD_minrank(g); r < GD_maxrank(g); r++) {
+	for (int i = 0; i < GD_rank(g)[r].n; i++) {
+	  node_t *const v = GD_rank(g)[r].v[i];
+	  for (int j = 0; (e = ND_out(v).list[j]); j++) {
              nname(v, stderr);
              fputs(" -> ", stderr);
              nname(aghead(e), stderr);
@@ -341,23 +377,18 @@ fixLabelOrder (graph_t* g, rank_t* rk)
 void
 checkLabelOrder (graph_t* g)
 {
-    int j, r, lo, hi;
     graph_t* lg = NULL;
-    rank_t* rk;
-    Agnode_t* u;
-    Agnode_t* n;
-    Agedge_t* e;
 
-    for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-	rk = GD_rank(g)+r;
-	for (j = 0; j < rk->n; j++) {
-	    u = rk->v[j];
-	    if ((e = ND_alg(u))) {
+    for (int r = GD_minrank(g); r <= GD_maxrank(g); r++) {
+	rank_t *const rk = GD_rank(g)+r;
+	for (int j = 0; j < rk->n; j++) {
+	    Agnode_t *const u = rk->v[j];
+	    if (ND_alg(u)) {
 		if (!lg) lg = agopen ("lg", Agstrictdirected, 0);
-		n = agnode(lg, ITOS(j), 1);
+		Agnode_t *const n = agnode(lg, ITOS(j), 1);
 		agbindrec(n, "info", sizeof(info_t), true);
-		lo = ND_order(aghead(ND_out(u).list[0]));
-		hi = ND_order(aghead(ND_out(u).list[1]));
+		int lo = ND_order(aghead(ND_out(u).list[0]));
+		int hi = ND_order(aghead(ND_out(u).list[1]));
 		if (lo > hi) {
 		    SWAP(&lo, &hi);
 		}
@@ -401,14 +432,11 @@ int dot_mincross(graph_t *g) {
 
     init_mincross(g);
 
-    ints_t scratch = {0};
-
     size_t comp;
     for (nc = 0, comp = 0; comp < GD_comp(g).size; comp++) {
 	init_mccomp(g, comp);
-	const int64_t mc = mincross(g, 0, &scratch);
+	const int64_t mc = mincross(g, 0);
 	if (mc < 0) {
-	    ints_free(&scratch);
 	    return -1;
 	}
 	nc += mc;
@@ -418,9 +446,8 @@ int dot_mincross(graph_t *g) {
 
     /* run mincross on contents of each cluster */
     for (int c = 1; c <= GD_n_cluster(g); c++) {
-	const int64_t mc = mincross_clust(GD_clust(g)[c], &scratch);
+	const int64_t mc = mincross_clust(GD_clust(g)[c]);
 	if (mc < 0) {
-	    ints_free(&scratch);
 	    return -1;
 	}
 	nc += mc;
@@ -433,9 +460,8 @@ int dot_mincross(graph_t *g) {
     if (GD_n_cluster(g) > 0 && (!(s = agget(g, "remincross")) || mapbool(s))) {
 	mark_lowclusters(g);
 	ReMincross = true;
-	const int64_t mc = mincross(g, 2, &scratch);
+	const int64_t mc = mincross(g, 2);
 	if (mc < 0) {
-	    ints_free(&scratch);
 	    return -1;
 	}
 	nc = mc;
@@ -444,15 +470,16 @@ int dot_mincross(graph_t *g) {
 	    check_vlists(GD_clust(g)[c]);
 #endif
     }
-    ints_free(&scratch);
     cleanup2(g, nc);
     return 0;
 }
 
-static adjmatrix_t *new_matrix(size_t i, size_t j) {
+static adjmatrix_t *new_matrix(size_t initial_rows, size_t initial_columns) {
     adjmatrix_t *rv = gv_alloc(sizeof(adjmatrix_t));
-    rv->nrows = i;
-    rv->ncols = j;
+    const size_t bits = initial_rows * initial_columns;
+    const size_t bytes = bits / 8 + (bits % 8 == 0 ? 0 : 1);
+    uint8_t *const data = gv_alloc(bytes);
+    *rv = (adjmatrix_t){.nrows = initial_rows, .ncols = initial_columns, .data = data};
     return rv;
 }
 
@@ -583,7 +610,7 @@ static void ordered_edges(graph_t * g)
     }
 }
 
-static int64_t mincross_clust(graph_t *g, ints_t *scratch) {
+static int64_t mincross_clust(graph_t *g) {
     int c;
 
     if (expand_cluster(g) != 0) {
@@ -592,13 +619,13 @@ static int64_t mincross_clust(graph_t *g, ints_t *scratch) {
     ordered_edges(g);
     flat_breakcycles(g);
     flat_reorder(g);
-    int64_t nc = mincross(g, 2, scratch);
+    int64_t nc = mincross(g, 2);
     if (nc < 0) {
 	return nc;
     }
 
     for (c = 1; c <= GD_n_cluster(g); c++) {
-	const int64_t mc = mincross_clust(GD_clust(g)[c], scratch);
+	const int64_t mc = mincross_clust(GD_clust(g)[c]);
 	if (mc < 0) {
 	    return mc;
 	}
@@ -742,13 +769,13 @@ static void transpose(graph_t * g, bool reverse)
     } while (delta >= 1);
 }
 
-static int64_t mincross(graph_t *g, int startpass, ints_t *scratch) {
+static int64_t mincross(graph_t *g, int startpass) {
     const int endpass = 2;
     int maxthispass = 0, iter, trying, pass;
     int64_t cur_cross, best_cross;
 
     if (startpass > 1) {
-	cur_cross = best_cross = ncross(scratch);
+	cur_cross = best_cross = ncross();
 	save_best(g);
     } else
 	cur_cross = best_cross = INT64_MAX;
@@ -756,14 +783,14 @@ static int64_t mincross(graph_t *g, int startpass, ints_t *scratch) {
 	if (pass <= 1) {
 	    maxthispass = MIN(4, MaxIter);
 	    if (g == dot_root(g))
-		if (build_ranks(g, pass, scratch) != 0) {
+		if (build_ranks(g, pass) != 0) {
 		    return -1;
 		}
 	    if (pass == 0)
 		flat_breakcycles(g);
 	    flat_reorder(g);
 
-	    if ((cur_cross = ncross(scratch)) <= best_cross) {
+	    if ((cur_cross = ncross()) <= best_cross) {
 		save_best(g);
 		best_cross = cur_cross;
 	    }
@@ -785,7 +812,7 @@ static int64_t mincross(graph_t *g, int startpass, ints_t *scratch) {
 	    if (cur_cross == 0)
 		break;
 	    mincross_step(g, iter);
-	    if ((cur_cross = ncross(scratch)) <= best_cross) {
+	    if ((cur_cross = ncross()) <= best_cross) {
 		save_best(g);
 		if (cur_cross < Convergence * (double)best_cross)
 		    trying = 0;
@@ -799,7 +826,7 @@ static int64_t mincross(graph_t *g, int startpass, ints_t *scratch) {
 	restore_best(g);
     if (best_cross > 0) {
 	transpose(g, false);
-	best_cross = ncross(scratch);
+	best_cross = ncross();
     }
 
     return best_cross;
@@ -1028,28 +1055,26 @@ void rec_reset_vlists(graph_t * g)
  * are laid out one component at a time and these will necessarily have a
  * node on each rank from source to sink levels.
  */
-static Agraph_t*
-realFillRanks (Agraph_t* g, int rnks[], int rnks_sz, Agraph_t* sg)
-{
+static Agraph_t *realFillRanks(Agraph_t *g, bitarray_t *ranks, Agraph_t* sg) {
     int i, c;
     Agedge_t* e;
     Agnode_t* n;
 
     for (c = 1; c <= GD_n_cluster(g); c++)
-	sg = realFillRanks (GD_clust(g)[c], rnks, rnks_sz, sg);
+	sg = realFillRanks(GD_clust(g)[c], ranks, sg);
 
     if (dot_root(g) == g)
 	return sg;
-    memset (rnks, 0, sizeof(int)*rnks_sz);
+    bitarray_clear(ranks);
     for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
-	rnks[ND_rank(n)] = 1;
+	bitarray_set(ranks, ND_rank(n), true);
 	for (e = agfstout(g,n); e; e = agnxtout(g,e)) {
 	    for (i = ND_rank(n)+1; i <= ND_rank(aghead(e)); i++) 
-		rnks[i] = 1;
+		bitarray_set(ranks, i, true);
 	}
     }
     for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
-	if (rnks[i] == 0) {
+	if (!bitarray_get(*ranks, i)) {
 	    if (!sg) {
 		sg = agsubg (dot_root(g), "_new_rank", 1);
 	    }
@@ -1071,9 +1096,9 @@ static void
 fillRanks (Agraph_t* g)
 {
     int rnks_sz = GD_maxrank(g) + 2;
-    int *rnks = gv_calloc(rnks_sz, sizeof(int));
-    realFillRanks (g, rnks, rnks_sz, NULL);
-    free (rnks);
+    bitarray_t rnks = bitarray_new(rnks_sz);
+    realFillRanks(g, &rnks, NULL);
+    bitarray_reset(&rnks);
 }
 
 static void init_mincross(graph_t * g)
@@ -1144,8 +1169,6 @@ static void flat_search(graph_t * g, node_t * v)
 	    if (ED_weight(e) == 0)
 		continue;
 	    if (ND_onstack(aghead(e))) {
-		assert(flatindex(aghead(e)) < M->nrows);
-		assert(flatindex(agtail(e)) < M->ncols);
 		matrix_set(M, (size_t)flatindex(aghead(e)), (size_t)flatindex(agtail(e)));
 		delete_flat_edge(e);
 		i--;
@@ -1153,8 +1176,6 @@ static void flat_search(graph_t * g, node_t * v)
 		    continue;
 		flat_rev(g, e);
 	    } else {
-		assert(flatindex(aghead(e)) < M->nrows);
-		assert(flatindex(agtail(e)) < M->ncols);
 		matrix_set(M, (size_t)flatindex(agtail(e)), (size_t)flatindex(aghead(e)));
 		if (!ND_mark(aghead(e)))
 		    flat_search(g, aghead(e));
@@ -1270,7 +1291,7 @@ int install_in_rank(graph_t *g, node_t *n) {
  *	graphs such as trees are drawn with no crossings.  it tries searching
  *	in- and out-edges and takes the better of the two initial orderings.
  */
-int build_ranks(graph_t *g, int pass, ints_t *scratch) {
+int build_ranks(graph_t *g, int pass) {
     int i, j;
     node_t *n, *ns;
     edge_t **otheredges;
@@ -1309,26 +1330,26 @@ int build_ranks(graph_t *g, int pass, ints_t *scratch) {
 	    continue;
 	if (!MARK(n)) {
 	    MARK(n) = true;
-	    node_queue_push_back(&q, n);
-	    while (!node_queue_is_empty(&q)) {
-		node_t *n0 = node_queue_pop_front(&q);
+	    LIST_PUSH_BACK(&q, n);
+	    while (!LIST_IS_EMPTY(&q)) {
+		node_t *n0 = LIST_POP_FRONT(&q);
 		if (ND_ranktype(n0) != CLUSTER) {
 		    if (install_in_rank(g, n0) != 0) {
-		        node_queue_free(&q);
+		        LIST_FREE(&q);
 		        return -1;
 		    }
 		    enqueue_neighbors(&q, n0, pass);
 		} else {
 		    const int rc = install_cluster(g, n0, pass, &q);
 		    if (rc != 0) {
-		        node_queue_free(&q);
+		        LIST_FREE(&q);
 		        return rc;
 		    }
 		}
 	    }
 	}
     }
-    assert(node_queue_is_empty(&q));
+    assert(LIST_IS_EMPTY(&q));
     for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
 	GD_rank(Root)[i].valid = false;
 	if (GD_flip(g) && GD_rank(g)[i].n > 0) {
@@ -1340,9 +1361,9 @@ int build_ranks(graph_t *g, int pass, ints_t *scratch) {
 	}
     }
 
-    if (g == dot_root(g) && ncross(scratch) > 0)
+    if (g == dot_root(g) && ncross() > 0)
 	transpose(g, false);
-    node_queue_free(&q);
+    LIST_FREE(&q);
     return 0;
 }
 
@@ -1354,7 +1375,7 @@ void enqueue_neighbors(node_queue_t *q, node_t *n0, int pass) {
 	    e = ND_out(n0).list[i];
 	    if (!MARK(aghead(e))) {
 		MARK(aghead(e)) = true;
-		node_queue_push_back(q, aghead(e));
+		LIST_PUSH_BACK(q, aghead(e));
 	    }
 	}
     } else {
@@ -1362,7 +1383,7 @@ void enqueue_neighbors(node_queue_t *q, node_t *n0, int pass) {
 	    e = ND_in(n0).list[i];
 	    if (!MARK(agtail(e))) {
 		MARK(agtail(e)) = true;
-		node_queue_push_back(q, agtail(e));
+		LIST_PUSH_BACK(q, agtail(e));
 	    }
 	}
     }
@@ -1378,7 +1399,7 @@ static bool constraining_flat_edge(Agraph_t *g, Agedge_t *e) {
   return true;
 }
 
-DEFINE_LIST(nodes, node_t *)
+typedef LIST(node_t *) nodes_t;
 
 /* construct nodes reachable from 'here' in post-order.
 * This is the same as doing a topological sort in reverse order.
@@ -1396,7 +1417,7 @@ static void postorder(graph_t *g, node_t *v, nodes_t *list, int r) {
 	}
     }
     assert(ND_rank(v) == r);
-    nodes_append(list, v);
+    LIST_APPEND(list, v);
 }
 
 static void flat_reorder(graph_t * g)
@@ -1413,7 +1434,7 @@ static void flat_reorder(graph_t * g)
 	base_order = ND_order(GD_rank(g)[r].v[0]);
 	for (i = 0; i < GD_rank(g)[r].n; i++)
 	    MARK(GD_rank(g)[r].v[i]) = false;
-	nodes_clear(&temprank);
+	LIST_CLEAR(&temprank);
 
 	/* construct reverse topological sort order in temprank */
 	for (i = 0; i < GD_rank(g)[r].n; i++) {
@@ -1429,8 +1450,8 @@ static void flat_reorder(graph_t * g)
 		flat_e = ND_flat_out(v).list[j];
 		if (constraining_flat_edge(g, flat_e)) local_out_cnt++;
 	    }
-	    if ((local_in_cnt == 0) && (local_out_cnt == 0))
-		nodes_append(&temprank, v);
+	    if (local_in_cnt == 0 && local_out_cnt == 0)
+		LIST_APPEND(&temprank, v);
 	    else {
 		if (!MARK(v) && local_in_cnt == 0) {
 		    postorder(g, v, &temprank, r);
@@ -1438,12 +1459,12 @@ static void flat_reorder(graph_t * g)
 	    }
 	}
 
-	if (nodes_size(&temprank) > 0) {
+	if (!LIST_IS_EMPTY(&temprank)) {
 	    if (!GD_flip(g)) {
-		nodes_reverse(&temprank);
+		LIST_REVERSE(&temprank);
 	    }
 	    for (i = 0; i < GD_rank(g)[r].n; i++) {
-		v = GD_rank(g)[r].v[i] = nodes_get(&temprank, (size_t)i);
+		v = GD_rank(g)[r].v[i] = LIST_GET(&temprank, (size_t)i);
 		ND_order(v) = i + base_order;
 	    }
 
@@ -1452,8 +1473,8 @@ static void flat_reorder(graph_t * g)
 		v = GD_rank(g)[r].v[i];
 		if (ND_flat_out(v).list) {
 		    for (size_t j = 0; (e = ND_flat_out(v).list[j]); j++) {
-			if ( (!GD_flip(g) && ND_order(aghead(e)) < ND_order(agtail(e))) ||
-				 ( (GD_flip(g)) && (ND_order(aghead(e)) > ND_order(agtail(e)) ))) {
+			if ((!GD_flip(g) && ND_order(aghead(e)) < ND_order(agtail(e))) ||
+				  (GD_flip(g) && ND_order(aghead(e)) > ND_order(agtail(e)))) {
 			    assert(!constraining_flat_edge(g, e));
 			    delete_flat_edge(e);
 			    j--;
@@ -1467,7 +1488,7 @@ static void flat_reorder(graph_t * g)
 	/* else do no harm! */
 	GD_rank(Root)[r].valid = false;
     }
-    nodes_free(&temprank);
+    LIST_FREE(&temprank);
 }
 
 static void reorder(graph_t * g, int r, bool reverse, bool hasfixed)
@@ -1572,7 +1593,7 @@ static int local_cross(elist l, int dir)
     return cross;
 }
 
-static int64_t rcross(graph_t *g, int r, ints_t *Count) {
+static int64_t rcross(graph_t *g, int r) {
     int top, bot, max, i, k;
     node_t **rtop, *v;
 
@@ -1580,28 +1601,21 @@ static int64_t rcross(graph_t *g, int r, ints_t *Count) {
     max = 0;
     rtop = GD_rank(g)[r].v;
 
-    // discard any data from previous runs
-    ints_clear(Count);
+    int *Count = gv_calloc(GD_rank(Root)[r + 1].n + 1, sizeof(int));
 
     for (top = 0; top < GD_rank(g)[r].n; top++) {
 	edge_t *e;
 	if (max > 0) {
 	    for (i = 0; (e = ND_out(rtop[top]).list[i]); i++) {
 		for (k = ND_order(aghead(e)) + 1; k <= max; k++)
-		    cross += ints_size(Count) <= (size_t)k
-		             ? 0
-		             : ints_get(Count, (size_t)k) * ED_xpenalty(e);
+		    cross += Count[k] * ED_xpenalty(e);
 	    }
 	}
 	for (i = 0; (e = ND_out(rtop[top]).list[i]); i++) {
 	    int inv = ND_order(aghead(e));
 	    if (inv > max)
 		max = inv;
-	    const size_t inv_z = (size_t)inv;
-	    if (ints_size(Count) <= inv_z) {
-		ints_resize(Count, inv_z + 1, 0);
-	    }
-	    ints_set(Count, inv_z, ints_get(Count, inv_z) + ED_xpenalty(e));
+	    Count[inv] += ED_xpenalty(e);
 	}
     }
     for (top = 0; top < GD_rank(g)[r].n; top++) {
@@ -1614,11 +1628,11 @@ static int64_t rcross(graph_t *g, int r, ints_t *Count) {
 	if (ND_has_port(v))
 	    cross += local_cross(ND_in(v), -1);
     }
+    free(Count);
     return cross;
 }
 
-static int64_t ncross(ints_t *scratch) {
-    assert(scratch != NULL);
+static int64_t ncross(void) {
     int r;
 
     graph_t *g = Root;
@@ -1627,7 +1641,7 @@ static int64_t ncross(ints_t *scratch) {
 	if (GD_rank(g)[r].valid)
 	    count += GD_rank(g)[r].cache_nc;
 	else {
-	    const int64_t nc = GD_rank(g)[r].cache_nc = rcross(g, r, scratch);
+	    const int64_t nc = GD_rank(g)[r].cache_nc = rcross(g, r);
 	    count += nc;
 	    GD_rank(g)[r].valid = true;
 	}
@@ -1740,7 +1754,7 @@ static bool medians(graph_t * g, int r0, int r1)
     }
     for (i = 0; i < GD_rank(g)[r0].n; i++) {
 	n = v[i];
-	if ((ND_out(n).size == 0) && (ND_in(n).size == 0))
+	if (ND_out(n).size == 0 && ND_in(n).size == 0)
 	    hasfixed |= flat_mval(n);
     }
     return hasfixed;
